@@ -100,8 +100,14 @@ sendmsg_one(struct iovec *iov, size_t iov_len, const char *path)
     const struct msghdr msg = { .msg_name = &event_addr, .msg_iov = iov, .msg_iovlen = iov_len };
     ssize_t count = sendmsg(fd, &msg, 0);
     if (count < 0) {
-        perror("uevent_sender sendmsg_one: sendmsg failed");
-        abort();
+	if (errno == ECONNREFUSED) {
+	    /* client side closed its monitor underneath us, so clean up and ignore */
+	    unlink(event_addr.sun_path);
+	    close(fd);
+	    return;
+	}
+	perror("uevent_sender sendmsg_one: sendmsg failed");
+	abort();
     }
     /* printf("passed %zi bytes to event socket %s\n", count, path); */
     close(fd);
@@ -220,9 +226,6 @@ append_property(char *array, size_t size, size_t offset, const char *name, const
         abort();
     }
 
-    /* this is already true as snprintf always writes the NUL, but -fanalyzer complains about use-of-uninitialized-value */
-    array[offset + r] = '\0';
-
     /* include the NUL terminator in the string length, as we need to keep it as a separator between keys */
     return r + 1;
 }
