@@ -23,6 +23,6 @@ typedef struct _uevent_sender uevent_sender;
 
 uevent_sender *uevent_sender_open(const char *rootpath);
 void uevent_sender_close(uevent_sender * sender);
-void uevent_sender_send(uevent_sender * sender, const char *devpath, const char *action);
+void uevent_sender_send(uevent_sender * sender, const char *devpath, const char *action, const char *properties);
 
 #endif				/* __UEVENT_SENDER_H */
