@@ -1747,6 +1747,11 @@ r 0 ^@^^^`^@a\n";
   /* end of script */
   ASSERT_EOF;
 
+  /* waiting is a no-op */
+  success = umockdev_testbed_wait_script(fixture->testbed, "/dev/greeter", &error);
+  g_assert(success);
+  g_assert_no_error(error);
+
   close(fd);
 }
 
@@ -1830,6 +1835,68 @@ r 0 OK\n";
   close(fd);
 }
 
+static void
+t_testbed_script_replay_wait(UMockdevTestbedFixture * fixture, UNUSED_DATA)
+{
+  gboolean success;
+  GError *error = NULL;
+  char buf[1024];
+
+  static const char* test_script = "r 100 Hello \n\
+r 100 world\n";
+
+  umockdev_testbed_add_from_string(fixture->testbed,
+          "P: /devices/greeter\nN: greeter\n"
+          "E: DEVNAME=/dev/greeter\nE: SUBSYSTEM=tty\nA: dev=4:64\n", &error);
+  g_assert_no_error(error);
+
+  /* load it */
+  success = umockdev_testbed_load_script_from_string(fixture->testbed, "/dev/greeter", test_script, &error);
+  g_assert_no_error(error);
+  g_assert(success);
+
+  /* wait for it; this writes the output into the pipe buffer, and theoretically may block
+   * take the risk for the unit test, it's small enough */
+  success = umockdev_testbed_wait_script(fixture->testbed, "/dev/greeter", &error);
+  g_assert(success);
+  g_assert_no_error(error);
+
+  /* start communication */
+  int fd = g_open("/dev/greeter", O_RDWR, 0);
+  g_assert_cmpint(fd, >=, 0);
+
+  /* we get the full message in a single read */
+  g_assert_cmpint(read(fd, buf, sizeof buf), ==, 11);
+  g_assert_cmpint(memcmp(buf, "Hello world", 11), ==, 0);
+
+  close(fd);
+
+  /* device has no script any more */
+  success = umockdev_testbed_wait_script(fixture->testbed, "/dev/greeter", &error);
+  g_assert_false(success);
+  g_assert_error(error, G_FILE_ERROR, G_FILE_ERROR_NOENT);
+  g_clear_error(&error);
+
+  /* can load a new script into the same device after waiting */
+  success = umockdev_testbed_load_script_from_string(fixture->testbed, "/dev/greeter", "r 50 Again\n", &error);
+  g_assert_no_error(error);
+  g_assert(success);
+  success = umockdev_testbed_wait_script(fixture->testbed, "/dev/greeter", &error);
+  g_assert(success);
+  g_assert_no_error(error);
+  fd = g_open("/dev/greeter", O_RDWR, 0);
+  g_assert_cmpint(fd, >=, 0);
+  g_assert_cmpint(read(fd, buf, sizeof buf), ==, 5);
+  g_assert_cmpint(memcmp(buf, "Again", 5), ==, 0);
+  close(fd);
+
+  /* invalid device */
+  success = umockdev_testbed_wait_script(fixture->testbed, "/dev/invalid", &error);
+  g_assert_false(success);
+  g_assert_error(error, G_FILE_ERROR, G_FILE_ERROR_NOENT);
+  g_clear_error(&error);
+}
+
 static void
 t_testbed_script_replay_socket_stream(UMockdevTestbedFixture * fixture, UNUSED_DATA)
 {
@@ -1901,8 +1968,6 @@ t_testbed_script_replay_fuzz(UMockdevTestbedFixture * fixture, UNUSED_DATA)
 {
   gboolean success;
   GError *error = NULL;
-  g_autofree char *tmppath = NULL;
-  int fd;
   char buf[1024];
 
   static const char* test_script = "f 20 -\n\
@@ -1915,20 +1980,13 @@ r 0 OK\n";
           "E: DEVNAME=/dev/fuzzy\nE: SUBSYSTEM=tty\nA: dev=4:64\n", &error);
   g_assert_no_error(error);
 
-  /* write script into temporary file */
-  fd = g_file_open_tmp("test_script_fuzzy.XXXXXX", &tmppath, &error);
-  g_assert_no_error(error);
-  g_assert_cmpint(write(fd, test_script, strlen(test_script)), >, 10);
-  close(fd);
-
   /* load it */
-  success = umockdev_testbed_load_script(fixture->testbed, "/dev/fuzzy", tmppath, &error);
+  success = umockdev_testbed_load_script_from_string(fixture->testbed, "/dev/fuzzy", test_script, &error);
   g_assert_no_error(error);
   g_assert(success);
-  g_unlink (tmppath);
 
   /* start communication */
-  fd = g_open("/dev/fuzzy", O_RDWR | O_NONBLOCK, 0);
+  int fd = g_open("/dev/fuzzy", O_RDWR | O_NONBLOCK, 0);
   g_assert_cmpint(fd, >=, 0);
   errno = 0;
 
@@ -1964,8 +2022,6 @@ t_testbed_replay_evemu_events(UMockdevTestbedFixture * fixture, UNUSED_DATA)
 {
   gboolean success;
   GError *error = NULL;
-  int fd;
-  g_autofree char *tmppath = NULL;
   struct timeval tv_begin, tv_end;
   struct input_event ev;
   static const char* test_data = "E: 1234.500000 0000 0000 0\n"   /* SYN */
@@ -1980,20 +2036,13 @@ t_testbed_replay_evemu_events(UMockdevTestbedFixture * fixture, UNUSED_DATA)
           "E: DEVNAME=/dev/input/event1\nE: SUBSYSTEM=input\n", &error);
   g_assert_no_error(error);
 
-  /* write evemu events file */
-  fd = g_file_open_tmp("test_evemu.XXXXXX", &tmppath, &error);
-  g_assert_no_error(error);
-  g_assert_cmpint(write(fd, test_data, strlen(test_data)), ==, strlen(test_data));
-  close(fd);
-
   /* load it */
-  success = umockdev_testbed_load_evemu_events(fixture->testbed, "/dev/input/event1", tmppath, &error);
+  success = umockdev_testbed_load_evemu_events_from_string(fixture->testbed, "/dev/input/event1", test_data, &error);
   g_assert_no_error(error);
   g_assert(success);
-  g_unlink (tmppath);
 
   /* start communication */
-  fd = g_open("/dev/input/event1", O_RDONLY, 0);
+  int fd = g_open("/dev/input/event1", O_RDONLY, 0);
   g_assert_cmpint(fd, >=, 0);
 
   g_assert_cmpint(gettimeofday(&tv_begin, NULL), ==, 0);
@@ -2041,6 +2090,24 @@ t_testbed_replay_evemu_events(UMockdevTestbedFixture * fixture, UNUSED_DATA)
   g_assert_cmpint(ev.value, ==, 1);
   assert_delta_t(&tv_begin, &tv_end, 550);
 
+  /* "flush" the script by waiting for it */
+  success = umockdev_testbed_wait_script(fixture->testbed, "/dev/input/event1", &error);
+  g_assert(success);
+  g_assert_no_error(error);
+
+  /* can load a new script now */
+  success = umockdev_testbed_load_evemu_events_from_string(fixture->testbed, "/dev/input/event1",
+                                                           "E: 1239.200000 0001 0123 1\n", &error);
+  g_assert_no_error(error);
+  g_assert(success);
+
+  g_assert_cmpint(read(fd, &ev, sizeof(ev)), ==, sizeof(ev));
+  g_assert_cmpint(ev.input_event_sec, ==, 1239);
+  g_assert_cmpint(ev.input_event_usec, ==, 200000);
+  g_assert_cmpint(ev.type, ==, 1);
+  g_assert_cmpint(ev.code, ==, 0x123);
+  g_assert_cmpint(ev.value, ==, 1);
+
   close(fd);
 }
 
@@ -2389,6 +2456,8 @@ main(int argc, char **argv)
 	       t_testbed_script_replay_default_device, t_testbed_fixture_teardown);
     g_test_add("/umockdev-testbed/script_replay_override_default_device", UMockdevTestbedFixture, NULL, t_testbed_fixture_setup,
 	       t_testbed_script_replay_override_default_device, t_testbed_fixture_teardown);
+    g_test_add("/umockdev-testbed/script_replay_wait", UMockdevTestbedFixture, NULL, t_testbed_fixture_setup,
+	       t_testbed_script_replay_wait, t_testbed_fixture_teardown);
     g_test_add("/umockdev-testbed/script_replay_evdev_event_framing", UMockdevTestbedFixture, NULL, t_testbed_fixture_setup,
                t_testbed_script_replay_evdev_event_framing, t_testbed_fixture_teardown);
     g_test_add("/umockdev-testbed/script_replay_socket_stream", UMockdevTestbedFixture, NULL, t_testbed_fixture_setup,
