@@ -155,17 +155,16 @@ class Testbed(unittest.TestCase):
             counters[4] = device.get_sysfs_attr('idVendor')
             counters[5] = device.get_property('ID_INPUT')
 
-        syspath = self.testbed.add_device('pci', 'mydev', None, ['idVendor', '0815'], ['ID_INPUT', '1'])
-        self.assertNotEqual(syspath, None)
-
         # set up listener for uevent signal
         client = GUdev.Client.new(['pci'])
         client.connect('uevent', on_uevent, counter)
-
         mainloop = GLib.MainLoop()
 
-        # send a signal and run main loop for 0.5 seconds to catch it
-        self.testbed.uevent(syspath, 'add')
+        syspath = self.testbed.add_device('pci', 'mydev', None, ['idVendor', '0815'], ['ID_INPUT', '1'])
+        self.assertNotEqual(syspath, None)
+
+        # Run the main loop for 0.5 seconds to catch the "add" uevent, which
+        # should be automatically generated by the call to `add_device()`.
         GLib.timeout_add(500, mainloop.quit)
         mainloop.run()
         self.assertEqual(counter, [1, 0, 0, syspath, '0815', '1'])
@@ -180,6 +179,72 @@ class Testbed(unittest.TestCase):
         mainloop.run()
         self.assertEqual(counter, [0, 0, 1, syspath, '0815', '1'])
 
+        counter[2] = 0
+        counter[3] = None
+        counter[4] = None
+        counter[5] = None
+
+        self.testbed.remove_device(syspath)
+
+        # Run the main loop for 0.5 seconds to catch the "remove" uevent, which
+        # should be automatically generated by the call to `remove_device()`.
+        GLib.timeout_add(500, mainloop.quit)
+        mainloop.run()
+        self.assertEqual(counter, [0, 1, 0, syspath, None, '1'])
+
+    def test_remove_usb_parent_device(self):
+        '''testbed removing a USB parent device removes children'''
+
+        counter = {}
+
+        def on_uevent(client, action, device, counters):
+            key = device.get_sysfs_path()
+            counters.setdefault(key, [0, 0])
+            if action == 'add':
+                counters[key][0] += 1
+            elif action == 'remove':
+                counters[key][1] += 1
+
+        # set up listener for uevent signal
+        client = GUdev.Client.new(['usb'])
+        client.connect('uevent', on_uevent, counter)
+        mainloop = GLib.MainLoop()
+
+        parent_syspath = self.testbed.add_device('usb', 'myparentdev', None, ['idVendor', '0815'], ['ID_INPUT', '1'])
+        self.assertNotEqual(parent_syspath, None)
+
+        child1_syspath = self.testbed.add_device('usb', 'child1', parent_syspath, ['idVendor', '0815'], ['ID_INPUT', '1'])
+        self.assertNotEqual(child1_syspath, None)
+
+        child2_syspath = self.testbed.add_device('usb', 'child2', parent_syspath, ['idVendor', '0815'], ['ID_INPUT', '1'])
+        self.assertNotEqual(child2_syspath, None)
+
+        # Run the main loop for 0.5 seconds to catch the "add" uevent, which
+        # should be automatically generated by the call to `add_device()`.
+        GLib.timeout_add(500, mainloop.quit)
+        mainloop.run()
+        self.assertIn(parent_syspath, counter)
+        self.assertIn(child1_syspath, counter)
+        self.assertIn(child2_syspath, counter)
+        self.assertEqual(counter[parent_syspath], [1, 0])
+        self.assertEqual(counter[child1_syspath], [1, 0])
+        self.assertEqual(counter[child2_syspath], [1, 0])
+
+        # Just remove the parent, the children should also be removed.
+        self.testbed.remove_device(parent_syspath)
+
+        # Run the main loop for 0.5 seconds to catch the "remove" uevent, which
+        # should be automatically generated by the call to `remove_device()`.
+        GLib.timeout_add(500, mainloop.quit)
+        mainloop.run()
+        self.assertEqual(counter[parent_syspath], [1, 1])
+        self.assertEqual(counter[child1_syspath], [1, 1])
+        self.assertEqual(counter[child2_syspath], [1, 1])
+
+        # Verify that the parent syspath has been removed (and therefore the
+        # children syspaths have also been removed.)
+        self.assertFalse(os.path.exists(parent_syspath))
+
     def test_add_from_string(self):
         self.assertTrue(self.testbed.add_from_string('''P: /devices/dev1
 E: SIMPLE_PROP=1
